//
#include <Wire.h>
#include <RTClib.h>

#define DS3231_ADDRESS        0x68  ///< I2C address for DS3231
#define DS3231_TIME           0x00  ///< Time register
#define DS3231_STATUSREG      0x0F  ///< Status register

#if (ARDUINO >= 100)
 #include <Arduino.h> // capital A so it is error prone on case-sensitive filesystems
 // Macro to deal with the difference in I2C write functions from old and new Arduino versions.
 #define _I2C_WRITE write   ///< Modern I2C write
 #define _I2C_READ  read    ///< Modern I2C read
#else
 #include <WProgram.h>
 #define _I2C_WRITE send    ///< Legacy I2C write
 #define _I2C_READ  receive ///< legacy I2C read
#endif



static uint8_t bin2bcd (uint8_t val) { return val + 6 * (val / 10); }
static uint8_t read_i2c_register(uint8_t addr, uint8_t reg) {
  Wire.beginTransmission(addr);
  Wire._I2C_WRITE((byte)reg);
  Wire.endTransmission();

  Wire.requestFrom(addr, (byte)1);
  return Wire._I2C_READ();
}
static void write_i2c_register(uint8_t addr, uint8_t reg, uint8_t val) {
  Wire.beginTransmission(addr);
  Wire._I2C_WRITE((byte)reg);
  Wire._I2C_WRITE((byte)val);
  Wire.endTransmission();
}

class NEW_RTC_DS3231: public RTC_DS3231{
  public:
    void adjustHour(uint8_t hour){
      Wire.beginTransmission(DS3231_ADDRESS);
      Wire._I2C_WRITE(DS3231_TIME + 0x02); // start at location 2 which is hour
      Wire._I2C_WRITE(bin2bcd(hour));
      Wire.endTransmission();
      
      uint8_t statreg = read_i2c_register(DS3231_ADDRESS, DS3231_STATUSREG);
      statreg &= ~0x80; // flip OSF bit
      write_i2c_register(DS3231_ADDRESS, DS3231_STATUSREG, statreg);
      }
  };
